import sys
import typing as t
from pathlib import Path

if "flask" in sys.modules:
    from flask import Flask, url_for, send_from_directory
    from markupsafe import Markup
else:
    raise ImportError("Flask is not installed.")

from vite_transporter._html_tags import BodyContent, ScriptTag, LinkTag
from vite_transporter.helpers import Colr
from vite_transporter._headers import http_headers


class ViteTransporter:
    app: t.Optional[Flask]
    vt_root_path: Path

    cors_allowed_hosts: list = None

    def __init__(
            self, app: t.Optional[Flask] = None, cors_allowed_hosts: t.Optional[list] = None
    ) -> None:
        if app is not None:
            self.init_app(app, cors_allowed_hosts)

    def init_app(self, app: Flask, cors_allowed_hosts: t.Optional[list] = None) -> None:
        if app is None:
            raise ImportError("No app was passed in.")
        if not isinstance(app, Flask):
            raise TypeError("The app that was passed in is not an instance of Flask")

        self.app = app

        if cors_allowed_hosts is not None:
            self.cors_allowed_hosts = cors_allowed_hosts

        if "vite_transporter" in self.app.extensions:
            raise ImportError(
                "The app has already been initialized with vite-to-flask."
            )

        self.app.extensions["vite_transporter"] = self
        self.app.config["VTF_APPS"] = {}
        self.vt_root_path = Path(app.root_path) / "vt"

        if not self.vt_root_path.exists():
            print(
                f"{Colr.WARNING}{Colr.BOLD}vt folder not found, a new one was created.{Colr.END}{Colr.END}\n\r"
                f"{Colr.OKCYAN}{self.vt_root_path}{Colr.END}\n\r"
            )
            self.vt_root_path.mkdir()

        for folder in self.vt_root_path.iterdir():
            if folder.is_dir():
                self.app.config["VTF_APPS"].update({folder.name: folder})

        self._load_routes(app)
        self._load_context_processor(app)
        self._load_cors_headers(app, self.cors_allowed_hosts)

    def _load_routes(self, app: Flask) -> None:
        @app.route("/__vt/<vite_app>/<filename>")
        def __vt(vite_app: str, filename: str):
            return send_from_directory(self.vt_root_path / vite_app, filename)

    @staticmethod
    def _load_context_processor(app: Flask) -> None:
        @app.context_processor
        def vt_head_processor():
            def vt_head(vite_app: str) -> t.Any:
                vite_assets = Path(app.root_path) / "vt" / vite_app
                find_vite_js = vite_assets.glob("*.js")
                find_vite_css = vite_assets.glob("*.css")

                tags = []

                for file in find_vite_js:
                    print("found", file)
                    tags.append(
                        ScriptTag(
                            src=url_for("__vt", vite_app=vite_app, filename=file.name),
                            type_="module",
                        )
                    )

                for file in find_vite_css:
                    print("found", file)
                    tags.append(
                        LinkTag(
                            rel="stylesheet",
                            href=url_for("__vt", vite_app=vite_app, filename=file.name),
                        )
                    )

                return Markup("".join([tag.raw() for tag in tags]))

            return dict(vt_head=vt_head)

        @app.context_processor
        def vt_body_processor():
            def vt_body(
                    root_id: str = "root",
                    noscript_message: str = "You need to enable JavaScript to run this app.",
            ) -> t.Any:
                return BodyContent(root_id, noscript_message)()

            return dict(vt_body=vt_body)

    @staticmethod
    def _load_cors_headers(app: Flask, cors_allowed_hosts: t.Optional[list] = None) -> None:
        if cors_allowed_hosts:
            print(
                f"\n\r{Colr.WARNING}{Colr.BOLD}vite-transporter is disabling CORS restrictions for:"
                f"{Colr.END}{Colr.END}\n\r"
                f"{Colr.OKCYAN}{', '.join(cors_allowed_hosts)}{Colr.END}\n\r"
            )

            @app.after_request
            def after_request(response):
                response.headers["Access-Control-Allow-Origin"] = ", ".join(cors_allowed_hosts)
                response.headers["Access-Control-Allow-Headers"] = ", ".join(
                    http_headers
                )
                response.headers["Access-Control-Allow-Methods"] = "*"
                response.headers["Access-Control-Allow-Credentials"] = "true"
                return response



http_headers = (
    "Content-Type",
    "x-requested-with",
    "x-csrf-token",
    "x-auth-token",
    "Authorization",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
    "X-CSRF-TOKEN",
    "X-Auth-Token",
    "X-Requested-With",
    "Accept",
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Auth-Token",
)



import sys

from .helpers import Colr as _Colr
from .helpers import PyProjectConfig as _PyProjectConfig
from .helpers import _compile
from .parser import ArgumentParser as _ArgumentParser

__version__ = "0.4.1"


def _cli():
    pars = _ArgumentParser(prog="vtf", add_help=False)
    pars.add_argument(
        "--version", "-v", action="version", version=f"vite-to-flask {__version__}"
    )
    pars.add_argument("--help", "-h", action="help")

    subparsers = pars.add_subparsers()

    compile_parser = subparsers.add_parser("compile")
    compile_parser.set_defaults(compile=False)
    compile_parser.add_argument("-y", action="store_true")

    list_parser = subparsers.add_parser("list")
    list_parser.set_defaults(list=False)

    ls_parser = subparsers.add_parser("ls")
    ls_parser.set_defaults(list=False)

    with _PyProjectConfig() as pyproject:
        pypro: _PyProjectConfig = pyproject

        for vite_app in pypro.vite_apps:
            pars.vite_apps.append(
                {
                    "vite_app": vite_app,
                    "serve_app": pypro.vt_config.get("serve_app"),
                }
            )

        args = pars.parse_args()

        if hasattr(args, "compile"):
            _compile(
                pyproject,
                pars.vite_apps,
                replace=True if hasattr(args, "y") and args.y else False,
            )

            # exit after compiling
            sys.exit(0)

        if hasattr(args, "list") or hasattr(args, "ls"):
            print("")
            if not pars.vite_apps:
                print(
                    f" {_Colr.WARNING}No vite apps found in pyproject.toml{_Colr.END}"
                )
            else:
                for app in pars.vite_apps:
                    print(
                        f"{_Colr.OKGREEN}{app.get('vite_app')}/dist/assets{_Colr.END} "
                        f"{_Colr.BOLD}=>{_Colr.END} "
                        f"{_Colr.OKGREEN}{app.get('serve_app')}/vt/{app.get('vite_app')}/{_Colr.END}"
                    )
            print("")

            # exit after listing
            sys.exit(0)

    # print help if no command is given
    pars.print_help()



from typing import Optional

from markupsafe import Markup


class LinkTag:
    rel: str = None
    href: str = None
    sizes: str = None
    type: str = None
    hreflang: str = None

    _rel: str = None
    _href: str = None
    _sizes: str = None
    _type: str = None
    _hreflang: str = None

    def __init__(
        self,
        rel: str,
        href: Optional[str] = None,
        sizes: Optional[str] = None,
        type_: Optional[str] = None,
        hreflang: Optional[str] = None,
    ):
        self.rel = rel
        self.href = href
        self.sizes = sizes
        self.type = type_
        self.hreflang = hreflang

        self._rel = f'rel="{self.rel}" '
        self._href = f'href="{self.href}" ' if self.href is not None else ""
        self._sizes = f'sizes="{self.sizes}" ' if self.sizes is not None else ""
        self._type = f'type="{self.type}" ' if self.type is not None else ""
        self._hreflang = (
            f'hreflang="{self.hreflang}" ' if self.hreflang is not None else ""
        )

    def __repr__(self):
        return Markup(
            f"<LinkTag {self._rel}{self._href}{self._sizes}{self._type}{self._hreflang}>".replace(
                " >", ">"
            )
        )

    def __str__(self):
        return Markup(self._compile())

    def __call__(self, *args, **kwargs):
        return Markup(self._compile())

    def raw(self):
        return self._compile()

    def _compile(self):
        return f"<link {self._rel}{self._href}{self._sizes}{self._type}{self._hreflang}>".replace(
            " >", ">"
        )


class ScriptTag:
    src: str = None
    type: str = None
    async_: bool = False
    defer: bool = False
    crossorigin: str = None
    integrity: str = None
    nomodule: bool = False
    referrerpolicy: str = None

    _src: str = None
    _type: str = None
    _async: str = None
    _defer: str = None
    _crossorigin: str = None
    _integrity: str = None
    _nomodule: str = None
    _referrerpolicy: str = None

    def __init__(
        self,
        src: str,
        type_: Optional[str] = None,
        async_: bool = False,
        defer: bool = False,
        crossorigin: Optional[str] = None,
        integrity: Optional[str] = None,
        nomodule: bool = False,
        referrerpolicy: Optional[str] = None,
    ):
        self.src = src
        self.type = type_
        self.async_ = async_
        self.defer = defer
        self.crossorigin = crossorigin
        self.integrity = integrity
        self.nomodule = nomodule
        self.referrerpolicy = referrerpolicy

        self._src = f'src="{self.src}" '
        self._type = f'type="{self.type}" ' if self.type is not None else ""
        self._async = f'async="{str(self.async_).lower()}" ' if self.async_ else ""
        self._defer = f"defer " if self.defer else ""
        self._crossorigin = (
            f'crossorigin="{self.crossorigin}" ' if self.crossorigin is not None else ""
        )
        self._integrity = (
            f'integrity="{self.integrity}" ' if self.integrity is not None else ""
        )
        self._nomodule = f"nomodule " if self.nomodule else ""
        self._referrerpolicy = (
            f'referrerpolicy="{self.referrerpolicy}" '
            if self.referrerpolicy is not None
            else ""
        )

    def __repr__(self):
        return Markup(
            (
                f"<ScriptTag {self._src}{self._type}"
                f"{self._async}{self._defer}{self._crossorigin}"
                f"{self._integrity}{self._nomodule}{self._referrerpolicy}>"
            ).replace(" >", ">")
        )

    def __str__(self):
        return Markup(self._compile())

    def __call__(self, *args, **kwargs):
        return Markup(self._compile())

    def raw(self):
        return self._compile()

    def _compile(self):
        return (
            f"<script {self._src}{self._type}"
            f"{self._async}{self._defer}{self._crossorigin}"
            f"{self._integrity}{self._nomodule}{self._referrerpolicy}></script>"
        ).replace(" >", ">")


class BodyContent:
    div_id: str = None
    noscript_message: str = None

    def __init__(
        self,
        div_id: str = "root",
        noscript_message: str = "You need to enable JavaScript to run this app.",
    ):
        self.div_id = div_id
        self.noscript_message = noscript_message

    def __repr__(self):
        return (
            "BodyContent< "
            f"id = {self.div_id} "
            f"noscript = {self.noscript_message} "
            ">"
        )

    def __str__(self):
        return Markup(self._compile())

    def __call__(self, *args, **kwargs):
        return Markup(self._compile())

    def _compile(self):
        return Markup(
            f'<div id="{self.div_id}"></div>\n'
            f"<noscript>{self.noscript_message}</noscript>"
        )



import argparse

from vite_transporter.helpers import Colr


class ArgumentParser(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        super(ArgumentParser, self).__init__(*args, **kwargs)
        self.vite_apps: list[dict] = []

    def print_help(self, file=None):
        print(
            "\n\r"
            "Usage: vtf <option>"
            "\n\r\n\r"
            f" {Colr.OKCYAN}list, ls{Colr.END} => List all vite apps in pyproject.toml"
            "\n\r"
            f" {Colr.OKCYAN}compile (-y){Colr.END} => Attempt to compile all vite apps"
            "\n\r"
            f"  | {Colr.OKCYAN}-y{Colr.END} => Accept all prompts while compiling"
            "\n\r"
            f" {Colr.OKCYAN}-h, --help{Colr.END} => Show the help message and exit"
            "\n\r"
            f" {Colr.OKCYAN}-v, --version{Colr.END} => Show the version and exit"
        )
        print("")



from dotenv import load_dotenv
from quart import Quart, render_template, session

from vite_transporter.platform.quart import ViteTransporter

load_dotenv()


def create_app():
    app = Quart(__name__)
    app.secret_key = "quart_secret"
    ViteTransporter(app, cors_allowed_hosts=["http://127.0.0.1:5003"])

    @app.route("/")
    async def index():
        return await render_template("index.html")

    @app.route("/api")
    def api():
        return {"message": "Hello, World!"}

    @app.route("/api/session")
    def session_():
        session["message"] = "Hello, World!"
        return {"session_message": session["message"]}

    return app


def run():
    # used for pyqwe
    _app = create_app()
    _app.run(port=5000, debug=True)



import shlex
import shutil
import subprocess
import sys
from pathlib import Path
from tomllib import loads


class Colr:
    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKCYAN = "\033[96m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"
    END = "\033[0m"


class PyProjectConfig:
    cwd: Path
    pyproject: Path
    vt_config: dict

    npm_exec: str
    npx_exec: str
    serve_app: str
    vite_apps: list

    def __init__(self):
        self.cwd = Path.cwd()
        self.pyproject = self.cwd / "pyproject.toml"
        self.load_pyproject()

    def load_pyproject(self):
        if not self.pyproject.exists():
            raise FileNotFoundError("pyproject.toml not found.")

        pyproject_raw = loads(str(self.pyproject.read_text()))
        self.vt_config = pyproject_raw.get("tool", {}).get("vite_transporter", {})
        self.npm_exec = self.vt_config.get("npm_exec", "npm")
        self.npx_exec = self.vt_config.get("npx_exec", "npx")
        self.serve_app = self.vt_config.get("serve_app", "app")
        self.vite_apps = self.vt_config.get("vite_apps", [])

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return None


class NPXCommander:
    def __init__(self, workdir: Path, npx_binary: str = "npx"):
        self.npx_binary = npx_binary
        self.workdir = workdir

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return None

    def run(self, command: str):
        subprocess.run([self.npx_binary, *shlex.split(command)], cwd=self.workdir)


class NPMCommander:
    def __init__(self, workdir: Path, npm_binary: str = "npm"):
        self.npm_binary = npm_binary
        self.workdir = workdir

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return None

    def run(self, command: str):
        subprocess.run([self.npm_binary, *shlex.split(command)], cwd=self.workdir)


def _compile(pypro: PyProjectConfig, vite_apps: list[dict], replace: bool = False):
    pc: PyProjectConfig = pypro

    print("Compiling Vite apps...")
    vt_dir = pc.cwd / pc.serve_app / "vt"

    # Delete contents of vt_dir
    if vt_dir.exists():
        if not replace:
            prompt = input(
                f"Continuing will replace the contents of \n\r"
                f"{vt_dir} \n\r"
                f"Do you want to continue? (Y/n): "
            )
        else:
            prompt = "y"

        if prompt.lower() == "y" or prompt == "":
            for item in vt_dir.iterdir():
                if item.is_dir():
                    shutil.rmtree(item)
                else:
                    item.unlink()

        else:
            print("Operation aborted.")
            sys.exit(0)

    else:
        vt_dir.mkdir()

    # Create directories for vite apps
    for app in vite_apps:
        va_path = pc.cwd / app.get("vite_app")
        va_node_modules = va_path / "node_modules"
        va_dist = va_path / "dist"
        va_assets = va_dist / "assets"

        va_vt_path = vt_dir / app.get("vite_app")

        if not va_vt_path.exists():
            va_vt_path.mkdir()

        if not va_node_modules.exists():
            with NPMCommander(va_path, pc.npm_exec) as npm:
                npm.run("install")

        with NPXCommander(va_path, pc.npx_exec) as npx:
            npx.run("vite build --mode production")

        for item in va_assets.iterdir():
            print(f"{Colr.OKGREEN}Copying {item.name} to {va_vt_path}{Colr.END}")

            if item.suffix == ".js":
                with open(va_vt_path / item.name, "w") as f:
                    content = item.read_text()
                    f.write(content.replace("assets/", f"__vt/{app.get('vite_app')}/"))
            else:
                shutil.copy(item, va_vt_path / item.name)

        shutil.rmtree(va_dist)

    print("Compilation complete.")
